<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Compressor\Null;

// Import framework stuff
use Org\Mxchange\CoreFramework\Compressor\Compressor;
use Org\Mxchange\CoreFramework\Object\BaseFrameworkSystem;

// Load SPL stuff
use \InvalidArgumentException;

/**
 * Null compression and decompression class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class NullCompressor extends BaseFrameworkSystem implements Compressor {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor!
		parent::__construct(__CLASS__);
	}

	/**
	 * Create a new compressor channel based a given compression handler
	 *
	 * @return	$compressorInstance		An instance of this class
	 */
	public static final function createNullCompressor () {
		// Get new instance
		$compressorInstance = new NullCompressor();

		// Return the compressor instance
		return $compressorInstance;
	}

	/**
	 * Null compression stream
	 *
	 * @param	$streamData		Mixed non-object stream data
	 * @return	$streamData		The compressed stream data
	 * @throws	InvalidArgumentException	If the stream is not compressable or decompressable
	 */
	public function compressStream ($streamData) {
		// Validate parameter
		if (is_object($streamData) || is_resource($streamData)) {
			// Throw an exception
			throw new InvalidArgumentException(sprintf('streamData[]=%s cannot be compressed/decompressed', gettype($streamData)));
		}

		// Return the compressed stream
		return $streamData;
	}

	/**
	 * Null decompression stream
	 *
	 * @param	$streamData		Mixed non-object stream data
	 * @return	$streamData		The decompressed stream data
	 * @throws	InvalidArgumentException	If the stream is not compressable or decompressable
	 */
	public function decompressStream ($streamData) {
		// Validate parameter
		if (is_object($streamData) || is_resource($streamData)) {
			// Throw an exception
			throw new InvalidArgumentException(sprintf('streamData[]=%s cannot be compressed/decompressed', gettype($streamData)));
		}

		// Return the decompressed stream
		return $streamData;
	}

	/**
	 * Getter for the file extension of this compressor
	 *
	 * @return	$string		Returns always 'null'
	 */
	public final function getCompressorExtension () {
		return 'null';
	}

}
