<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Factory\Stack;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Factory\BaseFactory;
use Org\Mxchange\CoreFramework\Factory\Object\ObjectFactory;
use Org\Mxchange\CoreFramework\Registry\GenericRegistry;

// Import SPL stuff
use \InvalidArgumentException;
use \SplFileInfo;

/**
 * A factory class for file-based stacks
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FileStackFactory extends BaseFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a singleton (registry-based) StackableFile instance
	 *
	 * @param	$prefix				Class prefix
	 * @param	$stackName			Name of the stack
	 * @return	$stackInstance		An instance of a StackableFile class
	 */
	public static final function createFileStackInstance (string $prefix, string $stackName) {
		// Validate parameter
		//* NOISY-DEBUG */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('FILE-STACK-FACTORY: prefix=%s,stackName=%s - CALLED!', $prefix, $stackName));
		if (empty($prefix)) {
			// Throw IAE
			throw new InvalidArgumentException('Parameter "prefix" is empty');
		} elseif (empty($stackName)) {
			// Throw it again
			throw new InvalidArgumentException('Paramter "stackName" is empty');
		}

		// Construct file stack name
		$fileInfoInstance = new SplFileInfo(sprintf('%s%s/%s.%s',
			FrameworkBootstrap::getConfigurationInstance()->getConfigEntry('root_base_path'),
			FrameworkBootstrap::getConfigurationInstance()->getConfigEntry('base_file_stacks_path'),
			$stackName,
			FrameworkBootstrap::getConfigurationInstance()->getConfigEntry('file_stacks_extension')
		));

		// If there is no handler?
		if (GenericRegistry::getRegistry()->instanceExists($stackName . '_stack')) {
			// Get handler from registry
			$stackInstance = GenericRegistry::getRegistry()->getInstance($stackName . '_stack');
		} else {
			// Get the handler instance
			$stackInstance = ObjectFactory::createObjectByConfiguredName($prefix . '_' . $stackName . '_stack_class', array($fileInfoInstance, $prefix . '_' . $stackName));

			// Add it to the registry
			GenericRegistry::getRegistry()->addInstance($stackName . '_stack', $stackInstance);
		}

		// Return the instance
		//* NOISY-DEBUG */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('FILE-STACK-FACTORY: stackInstance=%s - EXIT!', $stackInstance->__toString()));
		return $stackInstance;
	}

}
