<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Factory\User;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Factory\BaseFactory;
use Org\Mxchange\CoreFramework\Registry\GenericRegistry;
use Org\Mxchange\CoreFramework\Request\Requestable;

/**
 * A factory class for socket registries
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class UserFactory extends BaseFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a singleton user instance for given request instance.
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @return	$userInstance		An instance of a user class (@TODO use actual interface name)
	 */
	public static final function createUserByRequest (Requestable $requestInstance) {
		// Get registry instance
		$registryInstance = GenericRegistry::getRegistry();

		// Do we have an instance in the registry?
		if ($registryInstance->instanceExists('user')) {
			// Then use this instance
			$userInstance = $registryInstance->getInstance('user');
		} else {
			// Probe on member instance
			try {
				// Get class name
				$className = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry('user_class');

				// Try to instance it
				$userInstance = call_user_func_array(array($className, 'createMemberByRequest'), array($requestInstance));
			} catch (UnexpectedGuestAccountException $e) {
				// Then try it with guest account
				$className = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry('guest_class');

				// Try to instance it
				$userInstance = call_user_func_array(array($className, 'createGuestByRequest'), array($requestInstance));
			}

			// Set the instance in registry for further use
			$registryInstance->addInstance('user', $userInstance);
		}

		// Return the instance
		return $userInstance;
	}

}
